VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "INIFileReader"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' INI File Reader
'
' 29 September 2001
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' Current Location Settings

Private CurFileName As String
Private CurSection As String

'------------------------------------------------------------------
' API's

Private Declare Function GetPrivateProfileSection Lib "kernel32" Alias "GetPrivateProfileSectionA" (ByVal lpAppName As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Private Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Private Declare Function WritePrivateProfileSection Lib "kernel32" Alias "WritePrivateProfileSectionA" (ByVal lpAppName As String, ByVal lpString As String, ByVal lpFileName As String) As Long
Private Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long


'------------------------------------------------------------------
' Filename to edit in

Public Property Get INIFile() As String
  INIFile = CurFileName
End Property

Public Property Let INIFile(NewValue As String)
  CurFileName = NewValue
End Property


'------------------------------------------------------------------
' Section in INI File

Public Property Get Section() As String
  Section = CurSection
End Property

Public Property Let Section(NewValue As String)
  CurSection = NewValue
End Property


'------------------------------------------------------------------
' Get/Set Value

Public Property Get Value(Entry As String, Optional DefaultValue As String) As String
  Dim Buffer As String   ' Buffer for API return value
  Dim RetVal As Long     ' Return value
  
  If Not FileExist(CurFileName) Then
    Error 53
  Else
    Buffer = Space(200)
    RetVal = GetPrivateProfileString(CurSection, Entry, DefaultValue, Buffer, Len(Buffer), CurFileName)
    If RetVal = 0 Then
      Err.Raise 12345, "INI Reader", "Could not get ini entry value!"
    Else
      Value = Left(Buffer, RetVal)
    End If
  End If
End Property

Public Property Let Value(Entry As String, Optional DefaultValue As String, NewValue As String)
  Dim RetVal As Long
  RetVal = WritePrivateProfileString(CurSection, Entry, NewValue, CurFileName)
  If RetVal = 0 Then
    If WritePrivateProfileString(CurSection, Entry, DefaultValue, CurFileName) = 0 Then _
      Err.Raise 12345, "INI Reader", "Could not set ini entry value!"
  End If
End Property


'------------------------------------------------------------------
' Returns/Sets all entries in a section of a ini file

Public Property Get Values() As Variant
  Dim Buffer As String  ' Buffer for C-Array
  Dim RetVal As Long    ' Len of buffer
  
  If Not FileExist(CurFileName) Then
    Error 53
  Else
    Buffer = Space(200)
    RetVal = GetPrivateProfileSection(CurSection, Buffer, Len(Buffer), CurFileName)
    If RetVal = 0 Then
      Err.Raise 12345, "INI Reader", "Could not get ini section values!"
    Else
      Buffer = Left(Buffer, RetVal)
      Values = CVBArray(Buffer)
    End If
  End If
End Property

Public Property Let Values(NewSection As Variant)
  Dim RetVal As Long
  Dim CArray As String
  CArray = CCArray(NewSection)
  If WritePrivateProfileSection(CurSection, CArray, CurFileName) = 0 Then _
    Err.Raise 12345, "INI Reader", "Could not set ini file section!"
End Property


'------------------------------------------------------------------
' File Helpers

Private Function FileExist(FileName As String) As Boolean
  FileExist = Len(Dir(FileName)) And CBool(Len(FileName))
End Function


'------------------------------------------------------------------
' Conversion Routines from VB Arrays to C++ String 'Arrays'

Private Function CCArray(VBarray As Variant) As String
  Dim CArray As String
  Dim I As Integer
  
  For I = LBound(VBarray) To UBound(VBarray)
    CArray = CArray & VBarray(I) & vbNullChar
  Next
  CCArray = CArray
End Function

Private Function CVBArray(ByVal CArray As String) As Variant
  Dim VBarray As Variant
  Dim Pos As Long
  Dim NumFound As Integer
  NumFound = 0
  ReDim VBarray(NumFound) As Variant
  CArray = RTrim(CArray)
  
  Do
    If Right(CArray, 1) = vbNullChar Then CArray = Left(CArray, Len(CArray) - 1)
  Loop While Pos
  
  CArray = CArray & vbNullChar
  
  Do
    Pos = InStr(1, CArray, vbNullChar)
    If Pos Then
      ReDim Preserve VBarray(NumFound) As Variant
      VBarray(NumFound) = CVar(Left(CArray, Pos - 1))
      CArray = Mid(CArray, Pos + 1)
      NumFound = NumFound + 1
    End If
  Loop While Pos
  
  CVBArray = VBarray
End Function
